<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class Menu extends \Zeedhi\Framework\Controller\Simple {

    protected $priceController;
    protected $menuService;
    protected $storeService;
    protected $snackMessage;
    protected $util;

    public function __construct(\Controller\Price $priceController, \Service\MenuService $menuService, \Service\StoreService $storeService, \Util\SnackMessage $snackMessage, \Util\Util $util){
        $this->priceController = $priceController;
        $this->menuService = $menuService;
        $this->storeService = $storeService;
        $this->snackMessage = $snackMessage;
        $this->util = $util;
    }

    public function loadProducts(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL = $params[0]['value'];
            $CDLOJA = $params[1]['value'];
            $CDCAIXA = $params[2]['value'];
            $CDCLIENTE = $params[3]['value'];
            $imageUrl = $params[4]['value'];

            if ($CDCLIENTE == null){
                $CDCLIENTE = $this->storeService->getStandardClient($CDFILIAL);
            }

            /* Gets the products from the store. */
            $products = $this->menuService->getProducts($CDFILIAL, $CDCAIXA);

            foreach ($products as &$product) {
               $product['IMGGRUPO'] = $imageUrl.$product['IMGGRUPO'];
               $product['IMGPRODUCT'] = $imageUrl.$product['IMGPRODUCT'];
            }

            /* Gets the prices for products in the store. */
            $pricingTable = $this->priceController->getPricingTable($CDCLIENTE, $CDFILIAL, $CDLOJA);
            $effectiveDate = $this->priceController->checkEffective($pricingTable['CDFILTABPREC'], $pricingTable['CDTABEPREC']);
            $prices = $this->priceController->getPrices($CDFILIAL, $pricingTable['CDTABEPREC'], $effectiveDate);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('PriceRepository', $prices));


            /* Sets the prices. */
            $pricedProducts = array();
            foreach ($products as $product){
                try {
                    if ($product['PRICE'] !== '0,01' || $product['PRICE'] !== '0.01') {
                        $product['PRICE'] = floatval(number_format($prices[$product['CDPRODUTO']]['VRPRECITEM'], 2, '.', ','));
                        $product['IDIMPPRODUTO'] = $prices[$product['CDPRODUTO']]['IDIMPPRODUTO'];
                    }
                    if ($product['IDIMPPRODUTO'] == 2){
                        $product['PRICE'] = 0.01;
                    }
                    array_push($pricedProducts, $product);
                } catch (\Exception $e) {
                }
            }

            /* Get observations. They are sent in a separate datasource. */
            $observations = $this->menuService->getObservations($CDFILIAL, $CDLOJA);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductsRepository', $pricedProducts));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ObservationsRepository', $observations));

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function loadGrupProd(Request\Filter $request, Response $response){
        try{
            $params = $request->getFilterCriteria()->getConditions();

            $CDPRODUTO = $params[0]['value'];
            $grupo = $this->menuService->getGrupPromo($CDPRODUTO);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GrupoPromocaoCombinadaRepo', $grupo));
        } catch(\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function loadProdGrup(Request\Filter $request, Response $response){
        try{
            $params = $request->getFilterCriteria()->getConditions();

            $CDPRODPROMOCAO = $params[0]['value'];
            $CDGRUPROMOC = $params[1]['value'];
            $CDFILIAL = $params[2]['value'];
            $CDTABEPREC = $params[3]['value'];
            $DTINIVGPREC = $params[4]['value'];
            $priceLessProd = $this->menuService->getProdGrup($CDPRODPROMOCAO, $CDGRUPROMOC, $CDFILIAL, $CDTABEPREC, $DTINIVGPREC);
            $prodGrupo = array();
            foreach ($priceLessProd as $item){
                try {
                    $item['PRICE'] = floatval(number_format($item['PRICE'], 2, '.', ','));
                    if($item['VRDESC'] != null){
                        $item['VRDESC'] = floatval(number_format($item['VRDESC'], 2, '.', ','));
                        if ($item['TIPO'] == 'P') {
                            $item['VRDESC'] = 1 -($item['VRDESC']/100);
                            $item['PRICE'] = $item['PRICE']*$item['VRDESC'];
                        } else if ($item['TIPO'] == 'V') {
                            $item['PRICE'] = $item['PRICE']-$item['VRDESC'];
                        }
                    }
                    if($item['PRICE'] <= 0.01){
                        $item['PRICE'] = 0.01;
                    }

                    array_push($prodGrupo, $item);
                } catch (\Exception $e) {
                }
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProdutoDoGrupoRepo', $prodGrupo));
        } catch(\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

/*    public function loadProductsHistory(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();

             $CDCLIENTE    = $params[0]['value'];
             $CDCONSUMIDOR = $params[1]['value'];
             $CDFILIAL     = $params[2]['value'];
             $CDCAIXA      = $params[3]['value'];

            $historyArray = $this->menuService->getProductsHistory($CDCLIENTE, $CDCONSUMIDOR, $CDFILIAL, $CDCAIXA);
            // for is faster than forEach!
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductsHistoryRepo', $historyArray));

        } catch(\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }*/
}