<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class Order extends \Zeedhi\Framework\Controller\Simple {

    const MOVIMENTACAO_ENTRADA = 'E';
    const MOVIMENTACAO_SAIDA   = 'S';

    protected $productController;
    protected $orderService;
    protected $loginService;
    protected $storeService;
    protected $snackMessage;
    protected $util;

    public function __construct( \Controller\Product $productController, \Service\OrderService $orderService, \Service\Login $loginService, \Service\StoreService $storeService, \Util\SnackMessage $snackMessage, \Util\Util $util,\Service\SMTP $SMTPService){
        $this->productController       = $productController;
        $this->orderService            = $orderService;
        $this->loginService            = $loginService;
        $this->storeService            = $storeService;
        $this->snackMessage            = $snackMessage;
        $this->util                    = $util;
        $this->SMTPService             = $SMTPService;
    }

    public function makeOrderSchedule(Request\Filter $request, Response $response){
        try {

            /* Called from SendScheduleToProduction routine. */

            $params = $request->getFilterCriteria()->getConditions();

            $products   = json_decode($params[0]['value'], true);
            $payment    = json_decode($params[1]['value'], true);
            $clientData = json_decode($params[2]['value'], true);
            $CDFILIAL   = $params[3]['value'];
            $CDLOJA     = $params[4]['value'];
            $CDCAIXA    = $params[5]['value'];
            $ISTOGO     = $params[6]['value'];

            /* Make the order. */
            $currentOrder = $this->order($products, $payment, $clientData, $CDFILIAL, $CDLOJA, $CDCAIXA, true, $ISTOGO);

            //$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('MakeOrderRepository', $currentOrder));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function makeOrder(Request\Filter $request, Response $response){
        try {
            /* Entry point for standard orders. Arguments are passed from front end. */
            $params = $request->getFilterCriteria()->getConditions();
            $CDFILIAL          = $params[0]['value'];
            $CDLOJA            = $params[1]['value'];
            $CDCAIXA           = $params[2]['value'];

               try {

                /* Make the order. */
                $orderDetails = $this->order($CDFILIAL, $CDLOJA, $CDCAIXA);

                $qrCodeParams = array(
                    'CDFILIAL' => $orderDetails['ORDER']['CDFILIAL'],
                    'NRVENDAREST' => $orderDetails['ORDER']['NRVENDAREST'],
                    'NRCOMANDA' => $orderDetails['ORDER']['NRCOMANDA']
                );

            } catch (\Exception $e) {
                throw $e;
            }

            /* Send e-mail to the user. */
            //$this->prepareMail($orderDetails['ORDER'], $orderDetails['CONSUMER'], $orderDetails['PRODUCTS']);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('MakeOrderRepository', $orderDetails['ORDER']));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('QRCodeRepository', $qrCodeParams));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));

            /* In case it's an error with online payment, send error code to frontend handle it */
            if ($e->getCode() == '02') {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retorno', array('error' => $e->getCode())));
            }
        }
    }

    /* ORDERING FUNCTION, A.K.A. "PEDIDO" */
    public function order($CDFILIAL, $CDLOJA, $CDCAIXA){
        try {
            /* Validate products. */
            /* Validate consumer. */
            /* Get some required store-related details. */
            $generalParams = $this->storeService->getGeneralParams($CDFILIAL, $CDLOJA, $CDCAIXA);
            /* Open the bill. */
            /* Deal with money, save the payments and work out change. */
            $orderDetails = $this->openOrderBill($generalParams);
            /* Return to the main function. */
            return array(
                'ORDER' => $orderDetails
            );
            /* THAT'S IT! SUPER SIMPLE! */
        } catch (\Exception $e) {
            /* Send failure message to the main function. */
            throw new \Exception($e->getMessage());
        }
    }

    public function openOrderBill($generalParams){
        try {

            /* Gets the codes for the new order. */
            $nxtVendaRest = $this->util->newCode('VENDAREST'.$generalParams['CDFILIAL'], 10);
            $nxtComanda   = $this->util->newCode('COMANDAVEN'.$generalParams['CDFILIAL'], 10);
            /* Opens the order. */
            $this->orderService->insertProdutoVendaRest(
                $generalParams['CDFILIAL'],
                $nxtVendaRest,
                $generalParams['CDLOJA'],
                $generalParams['CDVENDEDORDLV'],
                $generalParams['CDOPERADOR']
            );
            /* Nulled values: IDSEXOCONS, CDPAIS*/
            $this->orderService->insertProdutoComandaVen(
                $generalParams['CDFILIAL'],
                $nxtVendaRest,
                $nxtComanda,
                $generalParams['CDLOJA']
            );

            /* Checks if the order was opened successfully. */
            return $this->checkOrder($generalParams['CDFILIAL'], 'DLV_'.$nxtComanda);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function dealWithMoney($generalParams, $consumerDetails, $payment, $nrVendaRest, $isSchedule){
        if (!$isSchedule) {
            $this->handleMoney($generalParams, $consumerDetails, $payment, $nrVendaRest);
        } else {
            $this->handleScheduleMoney($generalParams, $consumerDetails, $payment, $nrVendaRest);
        }
    }

    private function handleMoney ($generalParams, $consumerDetails, $payment, $nrVendaRest) {
        /* Finds out what is money. */
        $moneyCode = $this->storeService->getMoney();
        /* Inserts the payment types. */
        $change = 0;
        foreach ($payment['tipoRecebimento'] as $mov){
            if ($mov['CDTIPORECE'] === $moneyCode){
                $change = $mov['TROCO'] - $mov['VALOR'];
                $mov['VALOR'] = $mov['TROCO'];
            }
            $this->insertMovCaixa($generalParams['CDFILIAL'], $nrVendaRest, self::MOVIMENTACAO_ENTRADA, $consumerDetails['CDCLIENTE'], $consumerDetails['CDCONSUMIDOR'], $mov['CDTIPORECE'], $mov['VALOR']);
        }
        /* Deals with possible change. */
        if ($change > 0){
            $this->insertMovCaixa($generalParams['CDFILIAL'], $nrVendaRest, self::MOVIMENTACAO_SAIDA, $consumerDetails['CDCLIENTE'], $consumerDetails['CDCONSUMIDOR'], $moneyCode, $change);
        }
    }

    private function handleScheduleMoney ($generalParams, $consumerDetails, $payments, $nrVendaRest) {
        foreach ($payments as $payment) {
            $this->insertMovCaixa(
                $generalParams['CDFILIAL'],
                $nrVendaRest,
                $payment['IDTIPOMOVI'],
                $consumerDetails['CDCLIENTE'],
                $consumerDetails['CDCONSUMIDOR'],
                $payment['CDTIPORECE'],
                $payment['VRMOVI']
            );
        }
    }

    private function insertMovCaixa($codigoFilial, $nrVendaRest, $entradaSaida, $codigoCliente, $codigoConsumidor, $tipoRecebimento, $valor){
        $NRSEQMOVDLV = $this->util->newCode('MOVCAIXADLV'.$codigoFilial.$nrVendaRest, 10);
        $this->orderService->insertMovCaixaDlv($codigoFilial, $nrVendaRest, $NRSEQMOVDLV, $entradaSaida, $codigoCliente, $codigoConsumidor, $tipoRecebimento, $valor);
    }

    public function checkOrder($CDFILIAL, $DSCOMANDA){
        try {
            $orderDetails = $this->orderService->checkOrder($CDFILIAL, $DSCOMANDA);
            if (Empty($orderDetails)){
                throw new \Exception('Erro na validação da comanda.');
            }
            return $orderDetails;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getSequenceCode($CDFILIAL, $NRVENDAREST, $NRCOMANDA){
        try {
            $code = $this->orderService->getSequenceCode($CDFILIAL, $NRVENDAREST, $NRCOMANDA);
            return $code['NRSEQPRODCOM'];
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function changeOrderStatus(Request\Filter $request, Response $response){
        try {

        $params = $request->getFilterCriteria()->getConditions();
        $CDFILIAL = $params[0]['value'];
        $NRVENDAREST = $params[1]['value'];
        $NRCOMANDA = $params[2]['value'];

        $this->orderService->changeOrderStatus($CDFILIAL, $NRVENDAREST, $NRCOMANDA);

        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('QRCodeRepository'));

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function prepareMail($currentOrder, $consumerDetails, $assembledProducts){
        try {

            $NRCOMANDA = $currentOrder['NRCOMANDA'];
            $NMCONSUMIDOR = $consumerDetails['NMCONSUMIDOR'];
            $phoneNumber = $this->storeService->getStorePhone($currentOrder['CDFILIAL']);

            $subject  = $this->snackMessage->getMessage('21');
            $sender = $this->snackMessage->getMessage('19');
            $content =
                '<p><b>Ol&aacute; '. $NMCONSUMIDOR. '!</b></p>'.
                '<p>Seu pedido foi efetuado com sucesso!</p>'.
                '<p>O c&oacute;digo do seu pedido &eacute; ' . $NRCOMANDA . '!</p>'.
                ((!Empty($phoneNumber)) ? '<p>Para dúvidas ou reclamações, ligue para <b></b>.</p>' : '')
            ;

            $this->SMTPService->sendEmail($NMCONSUMIDOR, 'moises.freitas@odhen.com'/*$consumerDetails['DSEMAILCONS']*/, $sender, $subject, $content);

        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

}