<?php

namespace Service;

class DayDetails {

    protected $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager){
        $this->entityManager = $entityManager;
    }

    public function getGroupedProductsByDay($CDCLIENTE, $CDCONSUMIDOR, $DTINI, $DTFIM){
        $params = array(
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDCLIENTE' => $CDCLIENTE,
            ':DTINI' => $DTINI,
            ':DTFIM' => $DTFIM
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCTS_BY_DAY, $params);
    }

    public function getProductNutrients($CDPRODUTO,$FATORCOMPOSICAO){
        $nutrients = $this->getUnmergedProductNutrients($CDPRODUTO,$FATORCOMPOSICAO);
        $mergedNutrients = $this->mergeProductNutrients($nutrients);
        return $mergedNutrients;
    }

    public function calcProductNutrients($product, $nutrients, $buscaComposicao, $CDPRODUTO, $FATORCOMPOSICAO){
        if(isset($product[0]["VRPESOUNID"])){
            $productCompositionParam = $FATORCOMPOSICAO*$product[0]["VRPESOUNID"]*10;
            $productNutrients = $this->getNutrients($CDPRODUTO,$productCompositionParam);
            if(count($productNutrients) > 0){
                $buscaComposicao = false;
                $nutrients = array_merge($nutrients,$productNutrients);
            }
        }
        if($buscaComposicao){
            $subproducts = $this->getRotFabricaProduto($CDPRODUTO);
            foreach($subproducts as $key=>$subproduct){
                if (!empty($subproduct["CDSUBPRODUTO"])) {
                    $subproductFATORCOMPOSICAO = $FATORCOMPOSICAO * $subproduct["QTSUBPRODROT"];
                    $subproductCDPRODUTO =  $subproduct["CDSUBPRODUTO"];
                    $subproductNutrients = $this->getUnmergedProductNutrients($subproductCDPRODUTO, $subproductFATORCOMPOSICAO, true, $subproduct["QTSUBPRODROT"]);
                    $nutrients = array_merge($nutrients,$subproductNutrients);
                }
            }
        }
        return array($nutrients, $buscaComposicao);
    }

    public function getUnmergedProductNutrients($CDPRODUTO, $FATORCOMPOSICAO, $isSubproduct = false, $subProductProportion = 0.1){
        /*Fatorcomposicao é o multiplicador para acumular as quantidades
        de cada subproduto de um produto, de modo que o cálculo seja
        feito já juntando todas as quantidades.
        Inicialmente, recebe 1.*/
        $product = $this->getProduct($CDPRODUTO);
        $nutrients = array();
        $buscaComposicao = true;

        $isUnidadeKG = $product[0]["SGUNIDADE"] == "KG";
        $isUnidadeLT = $product[0]["SGUNIDADE"] == "LT";

        if((!$isUnidadeKG) && (!$isUnidadeLT)){
            $resultadoCalculoNutrientes = $this->calcProductNutrients($product, $nutrients, $buscaComposicao, $CDPRODUTO, $FATORCOMPOSICAO);
            $nutrients = $resultadoCalculoNutrientes[0];
            $buscaComposicao = $resultadoCalculoNutrientes[1];
        }else{
            if(!$isSubproduct){
                $resultadoCalculoNutrientes = $this->calcProductNutrients($product, $nutrients, $buscaComposicao, $CDPRODUTO, $FATORCOMPOSICAO);
                $nutrients = $resultadoCalculoNutrientes[0];
                $buscaComposicao = $resultadoCalculoNutrientes[1];
            }else{
                $productCompositionParam = $subProductProportion*10;
                $productNutrients = $this->getNutrients($CDPRODUTO,$productCompositionParam);
                if(count($productNutrients) > 0){
                    $buscaComposicao = false;
                    $nutrients = array_merge($nutrients,$productNutrients);
                }
            }
        }
        return $nutrients;
    }

    public function getRotFabricaProduto($CDPRODUTO){
        $params = array(
            ':CDPRODUTO' => $CDPRODUTO
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ROTFABRICA_PRODUTO,$params);
    }

    public function getProduct($CDPRODUTO){
        $params = array(
            ':CDPRODUTO' => $CDPRODUTO
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT, $params);
    }

    public function getNutrients($CDPRODUTO, $FATORCOMPOSICAO){
        $paramsNutri = array(
                    ':CDPRODUTO' => $CDPRODUTO,
                    ':FATORCOMPOSICAO' => $FATORCOMPOSICAO
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_NUTRIENTS, $paramsNutri);
    }
    public function getProductsNutritionalInformationsTotal($CDCLIENTE, $CDCONSUMIDOR, $DTMOVEXTCONS){
        $params = array(
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDCLIENTE' => $CDCLIENTE,
            ':DTMOVEXTCONS' => $DTMOVEXTCONS
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCTS_NUTRI_INFORMATIONS_TOTAL, $params);
    }

    public function mergeProductNutrients($nutrients){
        $mergedNutrients = array();
        foreach($nutrients as $key=>$nutrient){
            $CDNUTRIENTE = $nutrient["CDNUTRIENTE"];
            $VRNUTRIENTE = $nutrient["VRNUTRIENTE"];
            $NMNUTRIENTE = $nutrient["NMNUTRIENTE"];
            $SGUNIDADE = $nutrient["SGUNIDADE"];
            if(isset($mergedNutrients[$CDNUTRIENTE])){
                $mergedNutrients[$CDNUTRIENTE]["VRNUTRIENTE"] += $VRNUTRIENTE;
            }else{
                $mergedNutrients[$CDNUTRIENTE] = $nutrient;
            }
        }
        return array_values($mergedNutrients);
    }
}