<?php

namespace Odhen\API\Service;

class ImpressaoPedido {

    protected $entityManager;
    protected $impressaoUtil;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Odhen\API\Lib\ImpressaoUtil $impressaoUtil){
        $this->entityManager = $entityManager;
        $this->impressaoUtil = $impressaoUtil;
    }

    public function imprimePedido($CDFILIAL, $CDLOJA, $produtos, $CDVENDEDOR, $DSCOMANDA, $NRMESA, $CDSENHAPED, $tipoVenda, $toGoString){
        $this->impressaoUtil->delphiCriaArquivoImpressoras($CDFILIAL, $CDLOJA);
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA
        );
        $dadosLoja = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_LOJA, $params);
        $IDLUGARMESA = $dadosLoja[0]['IDLUGARMESA'];
        $IDPOSOBSPED = $dadosLoja[0]['IDPOSOBSPED'];
        $IDUTLCORTEPED = $dadosLoja[0]['IDUTLCORTEPED'];

        // TipoVenda B (balcão) do TAA não tem mesa.
        if (!empty($NRMESA)) {
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDLOJA' => $CDLOJA,
                'NRMESA' => $NRMESA
            );
            $dadosMesa = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_MESA, $params);
            $NMSALA = $dadosMesa[0]['NMSALA'];
        }
        else $NMSALA = null;

        $params = array(
            'CDVENDEDOR' => $CDVENDEDOR
        );
        $dadosVendedor = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_VENDEDOR, $params);

        if (!empty($dadosVendedor[0]['NMFANVEN'])) $NMFANVEN = $dadosVendedor[0]['NMFANVEN'];
        else $NMFANVEN = "";

        $produtosIndexadosPorImpressora = self::indexaProdutosPorImpressora($CDFILIAL, $CDLOJA, $produtos);
        $impressoraProducao = $produtosIndexadosPorImpressora['impressoraProducao'];
        $impressoraProducao2 = $produtosIndexadosPorImpressora['impressoraProducao2'];
        $impressoraPuxa = $produtosIndexadosPorImpressora['impressoraPuxa'];

        $result = array(
            'error' => false
        );
        foreach ($impressoraProducao as $impressoraAtual){
            $cutByPosition = $IDUTLCORTEPED === 'S';
            $respostaPedido = self::montaPedido($impressoraAtual['produtos'], $impressoraAtual['NRSEQIMPRLOJA'], $impressoraAtual['IDMODEIMPRES'], $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR, $NMFANVEN, $CDSENHAPED, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $impressoraAtual['tipoImpressao'], $toGoString, $cutByPosition);
            if ($respostaPedido['error']) $result = $respostaPedido;
        }
        foreach ($impressoraProducao2 as $impressoraAtual){
            $cutByPosition = $IDUTLCORTEPED === 'S';
            $respostaPedido = self::montaPedido($impressoraAtual['produtos'], $impressoraAtual['NRSEQIMPRLOJA'], $impressoraAtual['IDMODEIMPRES'], $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR, $NMFANVEN, $CDSENHAPED, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $impressoraAtual['tipoImpressao'], $toGoString, $cutByPosition);
            if ($respostaPedido['error']) $result = $respostaPedido;
        }
        foreach ($impressoraPuxa as $impressoraAtual){
            $respostaPedido = self::montaPedido($impressoraAtual['produtos'], $impressoraAtual['NRSEQIMPRLOJA'], $impressoraAtual['IDMODEIMPRES'], $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR, $NMFANVEN, $CDSENHAPED, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $impressoraAtual['tipoImpressao'], $toGoString, false);
            if ($respostaPedido['error']) $result = $respostaPedido;
        }
        return $result;
    }

    private function montaPedido($produtos, $NRSEQIMPRLOJA, $IDMODEIMPRES, $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR, $NMFANVEN, $CDSENHAPED, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $tipoImpressao, $toGoString, $cutByPosition){
        $ordemImp = 0;
        $text = '';

        $printerParams = $this->impressaoUtil->buscaParametrosImpressora($IDMODEIMPRES);
        if (($IDMODEIMPRES == '13') || ($IDMODEIMPRES == '16')) {
            $printerParams['largura'] = 42;
            $printerParams['tipoLetra'] = 3;
        }

        $text = self::printHeader($text, $printerParams, $tipoVenda, $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR, $NMFANVEN, $tipoImpressao, $toGoString);

        // Imprime pedido.
        $posicao = '';
        foreach ($produtos as $produtoAtual){
            if (!empty($produtoAtual['ORDEMIMP'])) {
                // Quando acabar os produtos da promoção, é impressa uma linha.
                if (($ordemImp != $produtoAtual['ORDEMIMP']) && ($ordemImp != 0)) {
                    $text .= $this->impressaoUtil->imprimeLinha($printerParams, $text);
                    $ordemImp = 0;
                }
            }

            // Imprime posição no pedido.
            if (($IDLUGARMESA === 'S') && ($tipoVenda === 'M')) {
                if ($posicao != $produtoAtual['NRLUGARMESA']) {
                    if ($posicao !== '' && $cutByPosition){ // Imprime por posição.
                        $text = self::printFooter($text, $printerParams, $CDSENHAPED);
                        $this->impressaoUtil->delphiImprimePedido($text, $NRSEQIMPRLOJA, $CDSENHAPED . $tipoImpressao, $IDMODEIMPRES);
                        $text = '';
                        $text = self::printHeader($text, $printerParams, $tipoVenda, $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR, $NMFANVEN, $tipoImpressao, $toGoString);
                    }
                    $text .= $printerParams['comandoEnter'];
                    $text .= 'POSICAO ' . $produtoAtual['NRLUGARMESA'] . $printerParams['comandoEnter'];
                    $posicao = $produtoAtual['NRLUGARMESA'];
                }
            }

            if (!empty($produtoAtual['IDIMPPROMOCAO'])) {
                // Cabeçalho da promoção inteligente.
                if ($produtoAtual['IDIMPPROMOCAO'] === 'S') {
                    $text .= $this->impressaoUtil->imprimeLinha($printerParams, $text);
                    $nomePromocao = isset($produtoAtual['NMPRODPROMOCAO']) ? $produtoAtual['NMPRODPROMOCAO'] : '-';
                    $text .= $this->impressaoUtil->centraliza($printerParams, $nomePromocao) . $printerParams['comandoEnter'];
                    if (!empty($produtoAtual['TXPRODCOMVEN'])){
                        // Imprime observação do produto pai.
                        $text .= $this->impressaoUtil->centraliza($printerParams, '* ' . $produtoAtual['TXPRODCOMVEN'] . $printerParams['comandoEnter']);
                        $produtoAtual['TXPRODCOMVEN'] = null;
                    }
                    $text .= $this->impressaoUtil->imprimeLinha($printerParams, $text);
                    // Variável usada para imprimir a última linha da promoção inteligente.
                    $ordemImp = $produtoAtual['ORDEMIMP'];
                }
            }

            if (!empty($produtoAtual['TXPRODCOMVEN'])) {
                // Imprime observação antes do produto.
                if (($produtoAtual['TXPRODCOMVEN'] !== " ") && ($IDPOSOBSPED === 'A') && ($produtoAtual['TXPRODCOMVEN'] != '')) {
                    $text .= '   *  ' . $produtoAtual['TXPRODCOMVEN'] . $printerParams['comandoEnter'];
                }
            }

            // Imprime observação do atraso no item antes do produto.
            if (($IDPOSOBSPED === 'A') && ($produtoAtual['ATRASOPROD'])) {
                $text .= '   *  Segura de ' . $produtoAtual['ATRASOPROD'] . ' minutos' . $printerParams['comandoEnter'];
            }

            if (!empty($produtoAtual['IDIMPPROMOCAO'])) {
                // Produto cabeçalho da promoção inteligente.
                if ($produtoAtual['IDIMPPROMOCAO'] !== 'S') {
                    $text .= ' ---> ' . $produtoAtual['QTPRODCOMVEN'] . ' ' . $produtoAtual['NMPRODUTO'] . $printerParams['comandoEnter'];
                }
            }

            if (!empty($produtoAtual['TXPRODCOMVEN'])) {
                // Imprime observação depois do produto.
                if (($produtoAtual['TXPRODCOMVEN'] !== " ") && ($IDPOSOBSPED === 'D') && ($produtoAtual['TXPRODCOMVEN'] != '')) {
                    $text .= '   *  ' . $produtoAtual['TXPRODCOMVEN'] . $printerParams['comandoEnter'];
                }
            }

            if (!empty($produtoAtual['ATRASOPROD'])) {
                // imprime observação do atraso no item depois do produto
                if (($IDPOSOBSPED === 'D') && ($produtoAtual['ATRASOPROD'])) {
                    $text .= '   *  Segura de ' . $produtoAtual['ATRASOPROD'] . ' minutos' . $printerParams['comandoEnter'];
                }
            }
        }

        $text = self::printFooter($text, $printerParams, $CDSENHAPED);
        return $this->impressaoUtil->delphiImprimePedido($text, $NRSEQIMPRLOJA, $CDSENHAPED . $tipoImpressao, $IDMODEIMPRES);
    }

    private function indexaProdutosPorImpressora($CDFILIAL, $CDLOJA, $produtos) {
        $impressoraProducao = array();
        $impressoraProducao2 = array();
        $impressoraPuxa = array();
        foreach ($produtos as $produtoAtual) {
            if (empty($produtoAtual['ORDEM'])) {
                $produtoAtual['ORDEM'] = 0;
            }
            if ($produtoAtual['IDTIPOCOMPPROD'] == '3'){
                    $produtoAtual['IDIMPPROMOCAO'] = 'S';
                    $params = array(
                        'CDFILIAL' => $CDFILIAL,
                        'CDLOJA' => $CDLOJA,
                        'CDPRODUTO' => $produtoAtual['CDPRODUTO']
                    );
            $impressoras = $this->entityManager->getConnection()->fetchAll(self::BUSCA_IMPRESSORAS, $params);
            self::buscaImpressoraPorProduto($impressoras, $produtoAtual, $CDFILIAL, $CDLOJA, $impressoraProducao, $impressoraProducao2, $impressoraPuxa);
            } else {
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDLOJA' => $CDLOJA,
                    'CDPRODUTO' => $produtoAtual['CDPRODUTO']
                );
                $impressoras = $this->entityManager->getConnection()->fetchAll(self::BUSCA_IMPRESSORAS, $params);
                self::buscaImpressoraPorProduto($impressoras, $produtoAtual, $CDFILIAL, $CDLOJA, $impressoraProducao, $impressoraProducao2, $impressoraPuxa);
            }
        }
        return array(
            'impressoraProducao' => $impressoraProducao,
            'impressoraProducao2' => $impressoraProducao2,
            'impressoraPuxa' => $impressoraPuxa
        );
    }

    private function buscaImpressoraPorProduto($impressoras, $produtoAtual, $CDFILIAL, $CDLOJA, &$impressoraProducao, &$impressoraProducao2, &$impressoraPuxa){
        if (!empty($produtoAtual['NRSEQIMPRLOJA'])) {
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDLOJA' => $CDLOJA,
                'NRSEQIMPRLOJA' => $produtoAtual['NRSEQIMPRLOJA']
            );
            $dadosImpressora = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_IMPRESSORA, $params);
            if (!empty($impressoraProducao[$dadosImpressora[0]['NRSEQIMPRLOJA']])) {
                array_push($impressoraProducao[$dadosImpressora[0]['NRSEQIMPRLOJA']]['produtos'], $produtoAtual);
            }
            else {
                $impressoraProducao[$dadosImpressora[0]['NRSEQIMPRLOJA']] = array(
                    'NRSEQIMPRLOJA' => $dadosImpressora[0]['NRSEQIMPRLOJA'],
                    'IDMODEIMPRES' => $dadosImpressora[0]['IDMODEIMPRES'],
                    'tipoImpressao' => 'P',
                    'produtos' => array($produtoAtual)
                );
            }
        } else if (!empty($impressoras[0]['A_NRSEQIMPRPROD'])) {
            if (!empty($impressoraProducao[$impressoras[0]['A_NRSEQIMPRPROD']])) {
                array_push($impressoraProducao[$impressoras[0]['A_NRSEQIMPRPROD']]['produtos'], $produtoAtual);
            }
            else {
                $impressoraProducao[$impressoras[0]['A_NRSEQIMPRPROD']] = array(
                    'NRSEQIMPRLOJA' => $impressoras[0]['A_NRSEQIMPRPROD'],
                    'IDMODEIMPRES' => $impressoras[0]['A_IDMODEIMPRES'],
                    'tipoImpressao' => 'P',
                    'produtos' => array($produtoAtual)
                );
            }
        }

        if (!empty($impressoras[0]['B_NRSEQIMPRPROD2'])) {
            if (!empty($impressoraProducao2[$impressoras[0]['B_NRSEQIMPRPROD2']])) {
                array_push($impressoraProducao2[$impressoras[0]['B_NRSEQIMPRPROD2']]['produtos'], $produtoAtual);
            }
            else {
                $impressoraProducao2[$impressoras[0]['B_NRSEQIMPRPROD2']] = array(
                    'NRSEQIMPRLOJA' => $impressoras[0]['B_NRSEQIMPRPROD2'],
                    'IDMODEIMPRES' => $impressoras[0]['B_IDMODEIMPRES'],
                    'tipoImpressao' => 'P',
                    'produtos' => array($produtoAtual)
                );
            }
        }

        if (!empty($impressoras[0]['C_NRSEQIMPRPUXA'])) {
            if (!empty($impressoraPuxa[$impressoras[0]['C_NRSEQIMPRPUXA']])) {
                array_push($impressoraPuxa[$impressoras[0]['C_NRSEQIMPRPUXA']]['produtos'], $produtoAtual);
            }
            else {
                $impressoraPuxa[$impressoras[0]['C_NRSEQIMPRPUXA']] = array(
                    'NRSEQIMPRLOJA' => $impressoras[0]['C_NRSEQIMPRPUXA'],
                    'IDMODEIMPRES' => $impressoras[0]['C_IDMODEIMPRES'],
                    'tipoImpressao' => 'U',
                    'produtos' => array($produtoAtual)
                );
            }
        }
        return array(
            'impressoraProducao' => $impressoraProducao,
            'impressoraProducao2' => $impressoraProducao2,
            'impressoraPuxa' => $impressoraPuxa
        );
    }

    private function printHeader($text, $printerParams, $tipoVenda, $DSCOMANDA, $NRMESA, $NMSALA, $CDVENDEDOR, $NMFANVEN, $tipoImpressao, $toGoString) {
        // Monta o cabeçalho do pedido.
        //********************************************************************************
        $text .= $this->impressaoUtil->imprimeLinha($printerParams, $text);
        $title = $tipoImpressao == 'P' ? 'PEDIDO' : 'RELATORIO DO PUXA';
        $text .= $this->impressaoUtil->centraliza($printerParams, $title) . $printerParams['comandoEnter'];
        $data = new \DateTime('NOW');
        $text .= $this->impressaoUtil->centraliza($printerParams, 'DATA: ' . $data->format('Y-m-d H:i:s')) . $printerParams['comandoEnter'];
        $text .= $this->impressaoUtil->imprimeLinha($printerParams, $text);
        if ($tipoVenda == 'C') {
            $text .= 'COMANDA.: ' . $DSCOMANDA . $printerParams['comandoEnter'];
        }
        if (!empty($NRMESA)) {
            $text .= 'MESA....: ' . $NRMESA . $printerParams['comandoEnter'];
        }
        if ($tipoVenda == 'M') {
            $text .= 'AMBIENTE: ' . $NMSALA . $printerParams['comandoEnter'];
        }
        if ($tipoVenda == 'B') {
            $text .= 'ORIGEM: TERMINAL DE AUTO ATENDIMENTO' . $printerParams['comandoEnter'];
        }
        $text .= 'GARCOM..: ' . $CDVENDEDOR . ' - ' . $NMFANVEN . $printerParams['comandoEnter'];
        $text .= $toGoString . $printerParams['comandoEnter'];
        $text .= $this->impressaoUtil->imprimeLinha($printerParams, $text);
        //********************************************************************************
        return $text;
    }

    private function printFooter($text, $printerParams, $CDSENHAPED) {
        // Monta rodapé.
        //********************************************************************************
        $text .= $this->impressaoUtil->imprimeLinha($printerParams, $text);
        $text .= $this->impressaoUtil->centraliza($printerParams, 'SENHA ' . $CDSENHAPED) . $printerParams['comandoEnter'];
        $text .= $this->impressaoUtil->imprimeLinha($printerParams, $text);
        $text .= $printerParams['comandoEnter'];
        $text .= $printerParams['comandoEnter'];
        $text .= $printerParams['comandoEnter'];
        $text .= $printerParams['comandoEnter'];
        $text .= $printerParams['comandoEnter'];
        //********************************************************************************
        return $text;
    }

    const BUSCA_DADOS_LOJA = "
        SELECT IDLUGARMESA, IDPOSOBSPED, IDUTLCORTEPED
          FROM LOJA
         WHERE CDFILIAL = :CDFILIAL
           AND CDLOJA = :CDLOJA
    ";

    const BUSCA_DADOS_VENDEDOR = "
        SELECT NMFANVEN
          FROM VENDEDOR
         WHERE CDVENDEDOR = :CDVENDEDOR
    ";

    const BUSCA_DADOS_MESA = "
        SELECT M.CDFILIAL, M.CDLOJA, M.NRMESA, S.NMSALA
          FROM MESA M JOIN SALA S
                        ON M.CDFILIAL = S.CDFILIAL
                       AND M.CDSALA = S.CDSALA
         WHERE M.CDFILIAL = :CDFILIAL
           AND M.CDLOJA = :CDLOJA
           AND M.NRMESA = :NRMESA
    ";

    const BUSCA_DADOS_IMPRESSORA = "
        SELECT P.NRSEQIMPRLOJA, I.IDMODEIMPRES
          FROM IMPRLOJA P, IMPRESSORA I
         WHERE P.CDFILIAL      = :CDFILIAL
           AND P.CDLOJA        = :CDLOJA
           AND P.NRSEQIMPRLOJA = :NRSEQIMPRLOJA
           AND P.CDIMPRESSORA  = I.CDIMPRESSORA
    ";

    const BUSCA_IMPRESSORAS = "
        SELECT A.NRSEQIMPRPROD AS A_NRSEQIMPRPROD, A.NRSEQIMPRLOJA AS A_NRSEQIMPRLOJA, A.NMIMPRLOJA AS A_NMIMPRLOJA,
               A.CDPORTAIMPR AS A_CDPORTAIMPR, A.IDMODEIMPRES AS A_IDMODEIMPRES,

               B.NRSEQIMPRPROD2 AS B_NRSEQIMPRPROD2, B.NRSEQIMPRLOJA AS B_NRSEQIMPRLOJA, B.NMIMPRLOJA AS B_NMIMPRLOJA,
               B.CDPORTAIMPR AS B_CDPORTAIMPR, B.IDMODEIMPRES AS B_IDMODEIMPRES,

               C.NRSEQIMPRPUXA AS C_NRSEQIMPRPUXA, C.NRSEQIMPRLOJA AS C_NRSEQIMPRLOJA, C.NMIMPRLOJA AS C_NMIMPRLOJA,
               C.CDPORTAIMPR AS C_CDPORTAIMPR, C.IDMODEIMPRES AS C_IDMODEIMPRES

          FROM

        (SELECT P.NRSEQIMPRPROD, L.NRSEQIMPRLOJA, L.NMIMPRLOJA, L.CDPORTAIMPR, I.IDMODEIMPRES
           FROM PRODLOJA P JOIN IMPRLOJA L
                             ON P.CDFILIAL = L.CDFILIAL
                            AND P.CDLOJA = L.CDLOJA
                            AND P.NRSEQIMPRPROD = L.NRSEQIMPRLOJA

                           JOIN IMPRESSORA I
                             ON L.CDIMPRESSORA = I.CDIMPRESSORA
          WHERE P.CDFILIAL = :CDFILIAL
            AND P.CDLOJA = :CDLOJA
            AND P.CDPRODUTO = :CDPRODUTO) A

        FULL JOIN

        (SELECT P.NRSEQIMPRPROD2, L.NRSEQIMPRLOJA, L.NMIMPRLOJA, L.CDPORTAIMPR, I.IDMODEIMPRES
           FROM PRODLOJA P JOIN IMPRLOJA L
                             ON P.CDFILIAL = L.CDFILIAL
                            AND P.CDLOJA = L.CDLOJA
                            AND P.NRSEQIMPRPROD2 = L.NRSEQIMPRLOJA

                           JOIN IMPRESSORA I
                             ON L.CDIMPRESSORA = I.CDIMPRESSORA
          WHERE P.CDFILIAL = :CDFILIAL
            AND P.CDLOJA = :CDLOJA
            AND P.CDPRODUTO = :CDPRODUTO) B

        ON 'A' = 'A' FULL JOIN

        (SELECT P.NRSEQIMPRPUXA, L.NRSEQIMPRLOJA, L.NMIMPRLOJA, L.CDPORTAIMPR, I.IDMODEIMPRES
           FROM PRODLOJA P JOIN IMPRLOJA L
                             ON P.CDFILIAL = L.CDFILIAL
                            AND P.CDLOJA = L.CDLOJA
                            AND P.NRSEQIMPRPUXA = L.NRSEQIMPRLOJA

                           JOIN IMPRESSORA I
                             ON L.CDIMPRESSORA = I.CDIMPRESSORA
          WHERE P.CDFILIAL = :CDFILIAL
            AND P.CDLOJA = :CDLOJA
            AND P.CDPRODUTO = :CDPRODUTO) C

        ON 'A' = 'A'
    ";

}