<?php
namespace Odhen\API\Util;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Doctrine\DBAL\Connection;
use Setforms2\Runtime\Helper\Procedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zend\Mail\Message as ZendMessage;
use Zend\Mail\Transport\Smtp as SmtpTransport;
use Zend\Mail\Transport\SmtpOptions;
use Zend\Mime\Mime as Mime;
use Zend\Mime\Part as MimePart;
use Zend\Mime\Message as MimeMessage;

class Util extends \Zeedhi\Framework\Controller\Simple {

    protected $entityManager;
    protected $instanceManager;
    protected $tipoBanco;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager) {
        $this->entityManager   = $entityManager;
        $this->instanceManager = $instanceManager;
        $this->tipoBanco       = $this->instanceManager->getParameter('connection_params')['driver'];
    }

    const ORACLE = 'oci8';

    public function logImpressao($info) {
        //testa se a pasta existe e cria caso não exista
        $folder = substr(__DIR__, 0, 1) . ":\\TEKNISA\\api\\logs\\";
        if (!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        //--grava o log
        $line = date("d-m-Y H:i:s") . " - " . $info . "\n";
        $drive = substr(__DIR__, 0, 1);
        $date = date("dmY");
        file_put_contents($drive . ":\\TEKNISA\\api\\logs\\logImpressao" . $date . ".txt", $line, FILE_APPEND);
    }

    public function testConnection(Request\Filter $request, Response $response) {
        try {
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('TestConnection', array(array('nothing' => 'nothing'))));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function httpRequest($URL, $params) {
        try {
            $paramsString = json_encode($params);
            $header = array(
                'Content-type: application/json; charset=utf-8'
            );

            $cURL = curl_init();
            curl_setopt($cURL, CURLOPT_URL, $URL);
            curl_setopt($cURL, CURLOPT_POSTFIELDS, $paramsString);
            curl_setopt($cURL, CURLOPT_HTTPHEADER, $header);
            curl_setopt($cURL, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
            curl_setopt($cURL, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($cURL, CURLOPT_TIMEOUT, 60);
            $httpResponse = curl_exec($cURL);
            $jsonResponse = json_decode($httpResponse, true);
            if ($httpResponse != false) {
                $result = array(
                    'error' => false,
                    'response' => $jsonResponse
                );
            } else {
                $result = array(
                    'error' => true,
                    'message' => 'Não foi possível completar a requisição HTTP. Erro: ' . curl_error($cURL)
                );
            }
        } catch (\Exception $e) {
            $result = array(
                'error' => true,
                'message' => 'Erro ao comunicar com o endereço: ' . $URL . '. Erro: ' . $e->getMessage()
            );
        }
        return $result;
    }

    public function getXMLParameter($key) {
        return $this->instanceManager->getParameter($key);
    }

    public function getConfigNFCEElement($fileName, $elementName){
        $result = array();
        $NFCEpath = $this->nfcePath();
        $pathFile = $NFCEpath.DIRECTORY_SEPARATOR."config".DIRECTORY_SEPARATOR.$fileName.".json";
        try{
            $fileContent = file_get_contents($pathFile);
            $arrContent = json_decode($fileContent, true);
            $result['error'] = false;
            $result['content'] = $arrContent[$elementName];
        }catch(\Exception $e){
            $result['error'] = true;
            $result['message'] = 'Falha ao encontrar o arquivo de configuração. Gentileza procurar o atendente mais próximo.';
        }
        return $result;
    }

    public function nfcePath(){
        $nfceSatPath = $this->getXMLParameter('PATH_NFCE_SAT');
        $nfcePath = $nfceSatPath.DIRECTORY_SEPARATOR.'NFCE';
        return $nfcePath;
    }
    const MASTER_PASSWORD = "s5vol0e00";
    public function sendMail($nome_destinatario, $email_destinatario, $mensagem) {
        require_once 'Zend/Mail.php';
        require_once 'Zend/Mail/Transport/Smtp.php';

        $date = date("d-m-Y");
        $drive = substr(__DIR__, 0, 1);

        // @todo - use DB parametrization

        $dominio = self::getXMLParameter('domain');

        $tr = new \Zend_Mail_Transport_Smtp(
            self::getXMLParameter('serverMail'),
            array(
                'auth'     => 'login',
                'username' => self::getXMLParameter('user'),
                'password' => self::getXMLParameter('password')
            )
        );

        \Zend_Mail::setDefaultTransport($tr);

        $mail = new \Zend_Mail('UTF-8');
        $mail->setFrom(self::getXMLParameter('user'), self::getXMLParameter('remetente'));
        $mail->setBodyHtml($mensagem, 'UTF-8');
        $mail->addTo($email_destinatario, $nome_destinatario);
        $mail->setSubject(self::getXMLParameter('assunto'));
        $mail->send();
    }

    public function encrypt($text, $salt = 'setforms') {
        return  trim(
            base64_encode(
                mcrypt_encrypt(
                    MCRYPT_RIJNDAEL_256,
                    $salt,
                    $text,
                    MCRYPT_MODE_ECB,
                    mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
                )
            )
        );
    }

    public function decrypt($text, $salt = 'setforms') {
        return trim(
            mcrypt_decrypt(
                MCRYPT_RIJNDAEL_256,
                $salt,
                base64_decode($text),
                MCRYPT_MODE_ECB,
                mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
            )
        );
    }

    public function delphiValidaSenhaOperador($CDSENHOPER_DIGITADA, $CDSENHOPER) {
        try {

            /* BYPASS */
            if ($CDSENHOPER_DIGITADA === self::MASTER_PASSWORD || $this->isDisabledCOM()){
                return array('error' => false, 'senhaOk' => true);
            }

            $dllCom = new \COM('Ifm.rdmPrint');
            $dllResponse = $dllCom->ValidaSenha($CDSENHOPER_DIGITADA, $CDSENHOPER);

            return $dllResponse == '1';
        } catch (\Exception $e) {
            throw new \Exception('Nao foi possivel carregar a IFM.dll, certifique-se que ela esta instalada.');
        }
    }

    public function delphiValidaPers($CDFILIAL, $CDCAIXA) {
        try {

            /* DISABLED FOR NOW (via Daniel). */
            return array('error' => false);

            if ($this->isDisabledCOM()) return array('error' => false);

            $dllCom = new \COM('Ifm.rdmPrint');
            if ($dllCom == '266') {
                $result = array(
                    'error' => true,
                    'message' => 'Nao foi possivel carregar a IFM.dll, certifique-se que ela esta instalada.'
                );
            } else {
                $resultado = $dllCom->ValidaPer($CDFILIAL, $CDCAIXA);
                if ($resultado == null) {
                    $result = array(
                        'error' => true,
                        'message' => 'Nao foi possivel carregar a IFM.dll, certifique-se que ela esta instalada.'
                    );
                } else if ($resultado != '1') {
                    $result = array(
                        'error' => true,
                        'message' => "Caixa não foi encontrado no arquivo de PERS."
                    );
                } else {
                    $result = array(
                        'error' => false
                    );
                }
            }
            return $result;
        } catch (\Exception $e) {
            return array(
                'error' => true,
                'message' => 'Nao foi possivel carregar a IFM.dll, certifique-se que ela esta instalada.'
            );
        }
    }

    public function createFolder($folder) {
        if (!is_dir($folder)) {
            if (mkdir($folder, 0777, true)) {
                return true;
            } else {
                return false;
            }
        } else {
            return true;
        }
    }

    // Remove this.
    public function databaseIsOracle() {
        return $this->tipoBanco == self::ORACLE;
    }

    public function geraCodigo($connection, $contador, $NRORG, $qtde, $size) {
    	try {
	        $referencia = "";
	        $procedure = new StoredProcedure($connection, 'NOVO_CODIGO');
	        $procedure->addParam(new Param('P_CONTADOR'  , Param::PARAM_INPUT , $contador  , Param::PARAM_TYPE_STR    ));
	        $procedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT, $referencia, Param::PARAM_TYPE_STR, 20));
	        $procedure->addParam(new Param('P_QTDE'      , Param::PARAM_INPUT , $qtde      , Param::PARAM_TYPE_INT    ));
	        if (self::databaseIsOracle()) {
	            $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $NRORG, Param::PARAM_TYPE_INT));
	        }
	        $procedure->execute();

	        $params = array(
	            'CDCONTADOR' => $contador
	        );
	        $sequencial = $connection->fetchAssoc(self::SQL_BUSCA_CONTADOR, $params);
	        $intSeq = (int)$sequencial['NRSEQUENCIAL'];
	        return str_pad($intSeq, $size, "0", STR_PAD_LEFT);
    	} catch(\Exception $e) {
    		throw new \Exception('Erro ao executar procedure de novo código.', 1);
    	}
    }


    const SQL_BUSCA_CONTADOR = "
        SELECT CDCONTADOR, NRSEQUENCIAL
          FROM NOVOCODIGO
         WHERE CDCONTADOR = :CDCONTADOR
    ";

    /**
     * @param $novocodigo
     * @param $tamanhoColuna
     * @param $numerico
     * @return int|string
     */
    private function preparaValor($novocodigo, $tamanhoColuna, $numerico) {
        /**
         * Valor do "$novoCodigo" vem uma string com "left zero fill" de tamanho 20, eg "00000000000012345678";
         * Por isso o casting para int, e o substr, lembrando de um exemplo onde o tamanho é 10:
         * substr($novocodigo, -1*$tamanhoColuna) => substr("00000000000012345678", -10) => "0012345678"
         * Lembrando que ele vai pegar os 10 ultimos digitos e/ou os 10 digitos menos significativos, que são os
         * 10 digitos a esquerda.
         */
        return $numerico === true ? (int)$novocodigo :  substr($novocodigo, -1*$tamanhoColuna);
    }

    /**
     * @param $stcodigo
     * @param $nrorg
     * @param $intervalo
     * @return mixed
     */
    private function executaProcedure($stcodigo, $nrorg, $intervalo, $commit = true) {
        // @todo usar o nrorg.
        if($commit) {
            $procedure = new Procedure('NOVO_CODIGO');
        } else {
            $procedure = new Procedure('NOVO_CODIGO_SC');
        }

        $procedure->addParam(new Param('P_CONTADOR', Param::PARAM_INPUT, $stcodigo, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 20));
        $procedure->addParam(new Param('P_QTDE', Param::PARAM_INPUT, $intervalo, Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $nrorg, Param::PARAM_TYPE_INT));
        $novocodigo = $procedure->call();
        return $novocodigo['P_SEQUENCIAL'];
    }

    // EMAIL VENDA -------------------------------------------------------------------

    public function sendEmailVenda($email_addressee, $content, $date, $nrseqvenda) {
        // Get data from server
        try {
            $sender     = "Teknisa";
            $subject    = "Cupom Fiscal Eletrônico";

            $SMTP       = $this->entityManager->getConnection()->fetchAll(Query::GET_SMTP);
            $username   = $SMTP[0]['DSEMAILAUVND'];
            $server     = $SMTP[0]['DSSMTPAUTVND'];
            $password   = $SMTP[0]['CDSENHAAUTVNDWEB'];
            $port       = $SMTP[0]['NRPORTAAUTVND'];
            $encryption = "null";
            $type       = "text/html; charset=UTF-8";
            $encoding   = "UTF-8";

            /* -- Dados Sapore
            $username   = "nfe-sapore@sapore.com.br";
            $server     = "mta.sapore.com.br";
            $password   = "S@p0r392";
            $port       = "25";
            $encryption = "null";
            */

            // Body e-mail.
            $allContent = self::generateEmail($subject, $content, $date, $nrseqvenda);
            $html       = new MimePart($allContent);
            $html->type = $type;
            $body       = new MimeMessage();
            $body->setParts(array($html));

            // E-mail config.
            $message = self::setConfigEmail($encoding, $email_addressee, $username, $sender, $subject, $body);

            // E-mail options.
            $options = self::setOptionsEmail($sender, $server, $port, $username, $password, $encryption);

            // Send e-mail.
            $transport = new SmtpTransport();
            $transport->setOptions($options);
            $transport->send($message);
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    const GET_SMTP = "
        SELECT DSSMTPAUTVND, DSEMAILAUVND, NRPORTAAUTVND, CDSENHAAUTVNDWEB,
               DSEMAILCLUSO
          FROM PARAMGERAL
    ";

    private function generateEmail($subject, $content, $date, $nrseqvenda) {
        $arrayOfSearch = Array('EMAIL_SUBJECT', 'EMAIL_CONTENT', 'EMAIL_NRSEQVENDA', 'EMAIL_DATE');
        $arrayOfReplace = Array($subject, $content, $nrseqvenda, $date);
        $docEmail = file_get_contents(__DIR__.'\..\Email\templateEmail.html');

        return str_replace($arrayOfSearch, $arrayOfReplace, $docEmail);
    }

    private function setConfigEmail($encoding, $email_addressee, $username, $sender, $subject, $body){
        $message = new ZendMessage();

        // -- for a while
        $name_addressee = $email_addressee;
        // --

        $message->setEncoding($encoding);
        $message->addTo($email_addressee, $name_addressee)
            ->addFrom($username, $sender)
            ->setSubject($subject)
            ->setBody($body);

        return $message;
    }

    private function setOptionsEmail($sender, $server, $port, $username, $password, $encryption){
        return new SmtpOptions(array(
            'name'              => $sender,
            'host'              => $server,
            'port'              => $port,
            'connection_class'  => 'login',
            'connection_config' => array(
                'username'    => $username,
                'password'    => $password,
                'encryption'  => $encryption
            )
        ));
    }

    // FIM EMAIL VENDA -------------------------------------------------------------------

    public function overthrow($P){
        try {
            $dllCom = new \COM('Ifm.rdmPrint');
            return $dllCom->DescriptografaSenha($P);
        } catch (\Exception $e) {
            throw new \Exception('Nao foi possivel carregar a IFM.dll, certifique-se que ela esta instalada.');
        }
    }

    public function removeMask($str) {
        return str_replace('-','', str_replace('.', '', str_replace('/', '', $str)));
    }

    public function aplicaMascaraCpfCnpj($value) {
    	$maskedValue = '';
    	if (strlen($value) > 11) {
    		$maskedValue = substr($value, 0, 2) . '.' . substr($value, 2, 3) . '.' . substr($value, 5, 3) . '/' . substr($value, 8, 4) . '-' .substr($value, 12, 2);
    	} else {
        	$maskedValue = substr($value, 0, 3) . '.' . substr($value, 3, 3) . '.' . substr($value, 6, 3) . '-' . substr($value, 9, 2);
    	}
    	return $maskedValue;
    }

    public function removeAllFiles($path) {
        $files = glob($path . '/*'); // get all file names
        foreach($files as $file) { // iterate files
            if (is_file($file)) {
                unlink($file); // delete file
            }
        }
    }

    /* Converte os valores numéricos retornados pelo banco em float */
    public function float($value) {
        return floatval(number_format($value, 2, '.', ','));
    }

    /***
     * Verifica se deve ou não utililzar o COM.
     */
    public function isDisabledCOM() {
        return $this->isDisabledDlls() || !$this->isWindowsEnvironment();
    }

    /***
     * Verifica se é ambiente Windows.
     */
    public function isWindowsEnvironment() {
        return defined("PHP_WINDOWS_VERSION_BUILD");
    }

    /***
     * Verifica se deve disabilitar as DLL e COM de acordo com o parametro 'disabled_dlls'.
     */
    public function isDisabledDlls() {
        try {
            $parameter = $this->getXMLParameter('disabled_dlls');
        } catch (\Exception $e) {
            $parameter = FALSE;
        }

        return $parameter;
    }

    public function getCOMTef() {
        try {
            $com = new \COM('TEF.TEFClass');
            // $com = new \COM('Tef.rdmTef');
            return array(
                'error' => false,
                'com'   => $com
            );
        } catch (\Exception $e) {
            return array(
                'error' => true,
                'message' => 'Erro ao inicializar DLL TEF.'
            );
        }
    }
}