function TabbedWidgetController($scope, templateManager, ScreenService, $timeout, eventEngine, ZHPromise) {

    var self = this;
    var WIDGET_ON_ACTIVATE_EVENT_NAME = 'WidgetOnActivate';

    $scope.tabbedWidget = $scope.$parent.widget;
    $scope.widgets = $scope.tabbedWidget.widgets;
    var tabs = getTabsList($scope.tabbedWidget);
    $scope.tabbedWidget.childTemplate = $scope.tabbedWidget.childTemplate || 'widget/form.html';

    var formTemplate = $scope.tabbedWidget.templateToForm;
    var viewTemplate = $scope.tabbedWidget.templateToView;

    $scope.tabbedWidget.templateToForm = 'widget/tabbedWidget.html';
    $scope.tabbedWidget.templateToView = 'widget/tabbedWidget.html';

    $scope.setDefaultTab = function(defaultTab) {
        return ZHPromise.when().then(function() {
            var tabbedWidget = $scope.tabbedWidget;
            if (!defaultTab) {
                if (tabbedWidget.widgets.length) {
                    defaultTab = tabbedWidget.widgets[0];
                }
            } else if (Util.isString(defaultTab)) {
                defaultTab = tabbedWidget.container.getWidget(defaultTab);
            }
            if (defaultTab) {
                tabbedWidget.currentWidget = defaultTab;
            }
            if (tabbedWidget.currentWidget) {
                tabbedWidget.currentWidget.activate();
            }
        });
    };

    var onActivateBoundEventIndex = eventEngine.bindEvent($scope.widget, WIDGET_ON_ACTIVATE_EVENT_NAME, onWidgetActivate);
    /**
     * The behaviour of the function onWidgetActivate should only happen when the widget is a tabbed.
     * After this controller is not used anymore, the event listener should be unbound
     */
    $scope.$on('$destroy', function() {
        eventEngine.unbindEvent($scope.widget, WIDGET_ON_ACTIVATE_EVENT_NAME, onActivateBoundEventIndex);
    });

    function getTabsList(tabbedWidget) {
        var tabs = tabbedWidget.widgets.slice(0);

        if (tabbedWidget.template.indexOf('tabbedWidget') === -1) {
            tabs.push(tabbedWidget);
        }

        return tabs;
    }

    function onWidgetActivate() {
        if ($scope.widget.currentWidget) {
            $scope.widget.currentWidget.activate();
        }
    }

    $scope.tabbedWidget.openTab = function(tabName) {
        var tabbed = $scope.tabbedWidget;
        var tab = getTabByName(tabName);
        $scope.onClickTab(tabbed, tab);
    };

    function getTabByName(name) {
        return tabs.find(function(widget) {
            return widget.name === name;
        });
    }

    $scope.onClickTab = function(tabbedWidget, childWidget) {
        var tabClickPromise;
        if (childWidget.isEnabled()) {
            tabClickPromise = closeGreaterSwipes(tabbedWidget, childWidget);
            tabbedWidget.currentWidget = childWidget;
            childWidget.activate();
        }
        return tabClickPromise;
    };

    $scope.isTabEnabled = function(tabbedWidget, childWidget) {
        return childWidget.isEnabled() && tabbedWidget.checkEnabledTab(childWidget);
    };

    $scope.widget.checkEnabledTab = function(childWidget) {
        if (Util.isUndefined(self._enabledTabs)) {
            self._enabledTabs = [];
        }

        return self._enabledTabs.length === 0 || self._enabledTabs.indexOf(childWidget) !== -1;
    };

    $scope.widget.setEnabledTabs = function(tabs) {
        tabs = Util.parseToArray(tabs);

        self._enabledTabs = [];
        for (var i = 0; i < tabs.length; i++) {
            if (Util.isString(tabs[i])) {
                tabs[i] = this.container.getWidget(tabs[i]);
            }
            self._enabledTabs.push(tabs[i]);
        }
        templateManager.updateTemplate();
    };

    $scope.widget.clearEnabledTabs = function() {
        this.setEnabledTabs();
    };

    function closeGreaterSwipes(tabbedWidget, childWidget) {
        var closeSwipePromise;
        if (tabbedWidget.currentWidget !== childWidget) {
            var levelToClose = ScreenService.getSwipeLevel(childWidget) + 1;
            if (ScreenService.isSwipeOpen(levelToClose)) {
                closeSwipePromise = ScreenService.closeSwipe(levelToClose).then(function() {
                    childWidget.activate();
                });
            }
        }
        return closeSwipePromise;
    }

    var updateTabbed = function(widget) {
        if (widget && widget.onEnterTab) {
            widget.onEnterTab();
        }
    };

    var buildChangeModeFunction = function(widget, cbk, template) {
        return function() {
            cbk();
            widget.childTemplate = template;
        };
    };

    if ($scope.tabbedWidget._tabbedMethodsUpdated === undefined) {
        var widget = $scope.tabbedWidget;
        widget.edit = buildChangeModeFunction(widget, widget.edit.bind(widget), formTemplate);
        widget.list = buildChangeModeFunction(widget, widget.list.bind(widget), formTemplate);
        widget.view = buildChangeModeFunction(widget, widget.view.bind(widget), viewTemplate);
        widget._tabbedMethodsUpdated = true;
    }

    $scope.$parent.$watch('widget.currentWidget', updateTabbed);

}