function MaskEngine(ZhUtil) {

	var FORMATER = "Format";
	var self = this;
	var PLAIN_TEXT_PASTE_TYPE = "text/plain";

	self.hasFormater = function(name) {
		return !!name && !!self[name + FORMATER];

	};

	self.getFormater = function(name) {
		if (!self.hasFormater(name)) {
			throw new Error("Formater '" + name + "' not found!");
		}
		return self[name + FORMATER];
	};

	self.numberFormat = function(number, params) {
		var defaultOptions = Util.extend($.currency.i18n[ZhUtil.getLang()], (params || {}), true);
		defaultOptions.symbol = "";
		defaultOptions.showSymbol = false;
		defaultOptions.precision = 0;
		defaultOptions.thousands = "";
		return self.currencyFormat(number, defaultOptions);
	};

	self.percentFormat = function(value, params) {
		var defaultParams = {
			precision: 2,
			minValue: 0,
			maxValue: 100
		};
		params = Util.extend(defaultParams, params);
		var percentualValue = (value - params.minValue) / (params.maxValue - params.minValue) * 100;
		return percentualValue.formatFloat(params.precision) + "%";
	};

	self.floatFormat = function(value, params) {
		var defaultOptions = Util.extend($.currency.i18n[ZhUtil.getLang()], (params || {}), true);
		defaultOptions.symbol = "";
		defaultOptions.showSymbol = false;
		return self.currencyFormat(value, defaultOptions);
	};

	self.currencyFormat = function(value, params) {
		var defaultOptions = Util.clone($.currency.i18n[ZhUtil.getLang()]);

		var valueFormated = "";
		try {
			var options = Util.extend(defaultOptions, (params || {}), true);
			valueFormated = $.currency.string(value, options);
		} catch (e) {
			valueFormated = value;
		}
		return valueFormated;
	};

	self.currency = function(element, viewValue, params, callBack) {
		var value = viewValue || '';
		element.val(value);
		params.callback = callBack;
		$.currency.setMask(params, element);
	};

	self.float = function(element, viewValue, params, callBack) {
		var value = viewValue || '';
		element.val(value);
		params.callback = callBack;

		params.symbol = '';
		params.showSymbol = false;
		$.currency.setMask(params, element);
	};

	self.date = function(element, viewValue, params, callBack) {
		$.mask.definitions.d = '[0-3]';
		$.mask.definitions.m = '[0-1]';
		var value = viewValue || '';
		element.val(value);
		element.mask("d9/m9/9999", {
			completed: function() {
				var value = this.val();
				callBack(value);
			}
		});
	};

	self.datetime = function(element, viewValue, params, callBack) {
		var DEFAULT_PARAM = "d9/m9/9999";
		var format = params.format || DEFAULT_PARAM;
		$.mask.definitions.d = '[0-3]';
		$.mask.definitions.m = '[0-1]';
		$.mask.definitions.h = '[0-2]';
		$.mask.definitions.s = '[0-5]';
		var value = viewValue;
		element.val(value);
		element.mask(format, {
			completed: function() {
				var value = this.val();
				callBack(value);
			}
		});
	};

	self.dateFormat = function(value, params) {
		params = params || {};
		var regSplit = new RegExp('\/|:|-', 'g');

		if (value && value.split(regSplit).length === 1) {
			value = parseInt(value);
		}

		return moment(value, params.inputFormat).format(params.format || "L LTS");
	};

	self.fix = function(element, viewValue, params, callBack) {
		if (element && params && params.mask) {
			var value = viewValue || '';
			element.val(value);
			params.mask = ZhUtil.evaluate(params.mask);
			element.mask(params.mask, {
				completed: function() {
					var value = this.val();
					callBack(value);
				}
			});

			return element.bind('keyup paste blur', function(event) {
					if (event.originalEvent) {
						handlePasteOnFixMask(event.originalEvent, element, params);
					}
					if (element.mask().length === 0) {
						callBack(element.mask());
					}
			});
		}
	};

	function handlePasteOnFixMask(event, element, params) {
		if (event.clipboardData && event.clipboardData.types && event.clipboardData.getData) {
			if (~event.clipboardData.types.indexOf(PLAIN_TEXT_PASTE_TYPE)) {
				pastedData = event.clipboardData.getData(PLAIN_TEXT_PASTE_TYPE);

				var maskAcceptedValues = getCleanFixMaskPattern(params.mask);
				var clearedPastedData = Util.clearValue(pastedData, { type: "fix", params: params }, self);

				if (maskAcceptedValues.length === Util.clearValue(clearedPastedData.length)) {
					element.val(pastedData);
				}
			}
		}
	}

	function getCleanFixMaskPattern(pattern) {
		var fixMaskDefinitions = 'a9*';
		return pattern.replace(new RegExp("[^" + fixMaskDefinitions + "]", "g"), "");
	}

	self.fixMaskParser = function(value, params) {
		var parsedValue = "";
		var maskPattern = params.mask;
		var patternsTranslation = Util.getFixMaskPatternsTranslation();

		if (value && maskPattern) {
			var j = 0;
			for (var i = 0; i < maskPattern.length; i++) {
				var currentDigit = value.charAt(i);
				var patternDigit = maskPattern[j];
				var patternTranslation = patternsTranslation[patternDigit];
				if (patternTranslation && patternTranslation.pattern.test(currentDigit)) {
					parsedValue += currentDigit;
				} else if (!patternTranslation && currentDigit !== patternDigit) {
					if(patternDigit){
						parsedValue += patternDigit;
					}
					parsedValue += currentDigit;
					j++;
				}
				j++;
			}
		} else {
			parsedValue = value;
		}

		return parsedValue;
	};

	self.fixParser = function fixParser(value, params) {
		var parsedValue = "";
		var maskPattern = params.mask;
		var patternsTranslation = Util.getFixMaskPatternsTranslation();

		if (value && maskPattern) {
			for (var i = 0; i < maskPattern.length; i++) {
				var currentDigit = value.charAt(i);
				var patternDigit = maskPattern[i];
				var patternTranslation = patternsTranslation[patternDigit];
				if (patternTranslation && patternTranslation.pattern.test(currentDigit)) {
					parsedValue += currentDigit;
				} else if (!patternTranslation && currentDigit !== patternDigit) {
					parsedValue += currentDigit;
				}
			}
		} else {
			parsedValue = value;
		}

		return parsedValue;
	};

	self.number = function(element, viewValue, params) {
		element.on("keyup", function() {
			element.val(self.numberFormat(element.val(), params));
		});

		element.val(self.numberFormat(viewValue, params));
	};

	var getLocaleSeparator = function() {
		var num = 1.1;
		return num.toLocaleString().slice(1, -1);
	};

	var _zerofill = function(value, length, params) {
		if (value) {
			value = value.toString();
			if (params.acceptOnlyNumbers) {
				value = value.replace(/[^0-9]/g, '');
			}
			while (value.length < length) {
				value = "0" + value;
			}
		}
		return value;
	};

	var _zerofillRight = function(value, length, params) {
		if (Util.isEmptyOrBlank(value)) {
			value = "0";
		}

		value = value.toString();
		if (params.acceptOnlyNumbers) {
			value = value.replace(/[^0-9]/g, '');
		}
		var decimalSeparator = value.indexOf(',') !== -1 ? ',' : (value.indexOf('.') !== -1 ? '.' : getLocaleSeparator());
		value = value.split(decimalSeparator);

		if (Util.isEmptyOrBlank(value[0])) {
			value[0] = "0";
		}

		if (value.length === 1) {
			value[1] = "";
		}

		if (value[1].length > length) {
			value[1] = value[1].slice(0, length);
		}
		while (value[1].length < length) {
			value[1] += "0";
		}

		value.splice(2);
		return value.join(decimalSeparator);
	};

	var _getZerofillLength = function(element, params) {
		if (params && params.maxlength) {
			return params.maxlength || 2;
		} else {
			return element ? element.attr('maxlength') : 4;
		}
	};

	var _removeLetters = function(e) {
		if(e.keyCode > 64 && e.keyCode < 91){
			return false;
		} else {
			return true;
		}
	};

	self.zerofillFormat = function(value, params, element) {
		params = params || {};
		var length = _getZerofillLength(element, params);
		if (params.right) {
			value = _zerofillRight(value, length, params);
		} else {
			value = _zerofill(value, length, params);
		}
		return value;
	};

	self.zerofill = function(element, viewValue, params, callBack) {
		if (viewValue) {
			var value = self.zerofillFormat(viewValue, params, element);
			element.val(value);
			callBack(value);
		}

		element.on('keydown', function(e){
			if(params.acceptOnlyNumbers){
				return _removeLetters(e);
			} else {
				return true;
			}
		});

		return element.bind('change', function() {
			var value = self.zerofillFormat($(this).val(), params, $(this));
			element.val(value);
			callBack(value);
		});
	};

	self.metric = function(element, viewValue, params) {
		element.on("blur", function() {
			element.val(self.metricFormat(element.val(), params));
		});
		element.val(self.metricFormat(viewValue, params));
	};

	var metricSuffixes = ['', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'];

	self.metricFormat = (function() {

		function metricFormat(value, params) {
			var opt = Util.clone($.currency.i18n[ZhUtil.getLang()]);
			params = $.extend(opt, params);

			var valueToReturn = "0";

			if (!Util.isEmptyOrBlank(value)) {
				if (Util.isString(value)) {
					var suffix = getSuffix(value);
					value = cleanValue(value, params);
					if (!isNaN(value)) {
						if (isMetricSuffix(suffix)) {
							value = metricDecode(value, suffix);
						}
						valueToReturn = setDecimals(metricEncode(value, params.precision), params.decimal);
					}
				} else if (!isNaN(value)) {
					valueToReturn = setDecimals(metricEncode(value, params.precision), params.decimal);
				} else if (Util.isNumber(value)) {
					valueToReturn = setDecimals(metricEncode(value, params.precision), params.decimal);
				}
			}
			return valueToReturn;
		}

		function metricEncode(value, precision) {
			for (var i = 0; value >= 1000; i++) {
				value /= 1000;
			}
			value = Util.roundDecimal(value, precision);
			return value + metricSuffixes[i];
		}

		function metricDecode(value, suffix) {
			for (var i = 0; metricSuffixes[i] != suffix; i++) {
				value *= 1000;
			}
			return value;
		}

		function cleanValue(value, params) {
			if (value) {
				value = value.toString();
				if (params.thousands) {
					var pattern = new RegExp("[" + params.thousands + "]", 'g');
					value = value.replace(pattern, '');
				}
				value = parseFloat(value.replace(params.decimal, '.').trim());
			}
			return value;
		}

		function setDecimals(value, decimal) {
			return value.replace(new RegExp('\\.', 'g'), decimal);
		}

		function getSuffix(value) {
			var suffix = value.substr(-1).toUpperCase();
			return suffix === 'K' ? 'k' : suffix;
		}

		function isMetricSuffix(suffix) {
			return metricSuffixes.indexOf(suffix) !== -1;
		}

		return metricFormat;

	})();

	this.timeFormatToSeconds = function(value, withSeconds) {
		var hours = "";
		var minutes = "";
		var seconds = "";
		var i = 0;

		while(value.charAt(i) !== "H"){
			hours = hours + value.charAt(i);
			i++;
		}
		i++;

		while(value.charAt(i) !== "m"){
			minutes = minutes + value.charAt(i);
			i++;
		}
		i++;

		if (withSeconds) {
			while(value.charAt(i) !== "s"){
				seconds = seconds + value.charAt(i);
				i++;
			}
		}

		hours = parseFloat(hours);
		minutes = parseFloat(minutes);
		seconds = parseFloat(seconds);

		var result = (hours * 3600) + (minutes * 60);

		if(!isNaN(seconds)){
			result += seconds;
		}

		return result;
	};

	this.secondsToTimeFormat = function(value, params) {
		value = parseFloat(value);
		var response = "";
		var withSeconds = params.withSeconds || false;


		var hours = parseInt(value / 3600);
		response += hours + "H";

		var minutes = parseInt((value % 3600 ) / 60);
		response += minutes + "m";

		var seconds = parseInt((value % 3600) % 60);
		if(seconds > 0 && withSeconds) {
			response += seconds + "s";
		}

		return response;
	};

	this.getMappedMasks = function() {
		return maskNameToMaskFunction;
	};

	this.hasMask = function(maskName) {
		return maskNameToMaskFunction.hasOwnProperty(maskName);
	};

	this.registerMask = function(maskName, maskFunction, maskFormatFunction) {
		if (maskName && maskFunction) {
			if (!maskNameToMaskFunction[maskName]) {
				maskNameToMaskFunction[maskName] = {};
				maskNameToMaskFunction[maskName].mask = maskFunction;
				maskNameToMaskFunction[maskName].format = maskFormatFunction;
			} else {
				throw ScreenService.i18n("Mask __maskName__ is already mapped!", {
					maskName: maskName
				});
			}
		} else {
			throw ScreenService.i18n("Invalid parameters");
		}
	};

	var maskNameToMaskFunction = {
		"currency": {
			"mask": self.currency,
			"format": self.currencyFormat
		},
		"float": {
			"mask": self.float,
			"format": self.floatFormat
		},
		"date": {
			"mask": self.date,
			"format": self.dateFormat
		},
		"datetime": {
			"mask": self.datetime,
			"format": undefined
		},
		"fix": {
			"mask": self.fix,
			"format": self.fixMaskParser,
			"parser": self.fixParser
		},
		"number": {
			"mask": self.number,
			"format": self.numberFormat
		},
		"zerofill": {
			"mask": self.zerofill,
			"format": self.zerofillFormat
		},
		"metric": {
			"mask": self.metric,
			"format": self.metricFormat
		},
		"percent": {
			"mask": undefined,
			"format": self.percentFormat
		},
		"time": {
			"mask": undefined,
			"format": self.secondsToTimeFormat
		}
	};
}