describe("ContainerMerger", function(){

	var defaultContainer = {
		"name": "login",
		"showFooter": true,
		"showMenu": true,
		"widgets": [
			{
				"id": 1,
				"name": "loginWidget",
				"template": "widget/list.html",
				"actions": [
					{
						"id": 2,
						"name": "TEST",
						"showAsAction": "always",
						"events": [
							{
								"id": 3,
								"name": "ActionEvent",
								"code": ";"
							}
						]
					}
				],
				"events": [
					{
						"id": 4,
						"name": "WidgetOnEnter",
						"code": ";"
					}
				],
				"fields": [
					{
						"id": 5,
						"name": "TEST",
						"label": "Default label"
					}
				],
				"styles": [
					{
						"id": 6,
						"condition": "row.id % 2 == 0",
						"cssClass": "zh-active"
					}
				],
				"fieldGroups": [
					{
						"id": 7,
						"name": "ORG",
						"label": "Dados da organização",
						"collapsible": true,
						"opened": true
					}
				],
				"widgets": [
					{
						"id": 100,
						"name": "secondWidget",
						"label": "Second widget",
						"events": [
							{
								"id": 102,
								"name": "WidgetOnEnter",
								"code": ";"
							}
						]
					}
				]
			}
		]
	};

	var defaultContainerPreference = {
		"name": "login",
		"showFooter": false,
		"widgets": [
			{
				"id": 1,
				"name": "loginWidget",
				"template": "widget/form.html",
				"actions": [
					{
						"id": 2,
						"name": "TEST",
						"showAsAction": "never",
						"events": [
							{
								"id": 3,
								"name": "ActionEvent",
								"code": "args.owner.widget.newRow();"
							}
						]
					}
				],
				"events": [
					{
						"id": 4,
						"name": "WidgetOnEnter",
						"code": "args.owner.newRow();"
					}
				],
				"fields": [
					{
						"id": 5,
						"name": "TEST",
						"label": "Preference label"
					},{
						"id": 20,
						"name": "NEW_FIELD",
						"label": "New field"
					}
				],
				"styles" : [
					{
						"id": 6,
						"condition" : "row.id % 2 === 0",
						"cssClass" : "zh-inactive"
					}
				],
				"fieldGroups": [
					{
						"id": 7,
						"name": "ORG changed",
						"label": "Dados da organização",
						"collapsible": true,
						"opened": false
					}
				],
				"widgets": [
					{
						"id": 1000,
						"name": "thirdWidget",
						"label": "3"
					},
					{
						"id": 100,
						"name": "secondWidget",
						"label": "2",
						"fields": [
							{
								"id": 101,
								"name": "second TEST",
								"label": "Second widget field label"
							}
						],
						"events": [
							{
								"id": 102,
								"name": "WidgetOnEnter",
								"code": "args.owner.newRow();"
							}
						]
					}
				]
			}
		]
	};

	var containerMerger = new ContainerMerger();

	it("The method merge should merge two containers", function() {
		var mergedContainer = containerMerger.merge(Util.clone(defaultContainer), Util.clone(defaultContainerPreference));
		expect(mergedContainer.showFooter).toEqual(false);
		expect(mergedContainer.showMenu).toEqual(true);
		expect(mergedContainer.widgets[0].template).toEqual('widget/form.html');
		expect(mergedContainer.widgets[0].actions[0].showAsAction).toEqual('never');
		expect(mergedContainer.widgets[0].actions[0].events[0].code).toEqual('args.owner.widget.newRow();');
		expect(mergedContainer.widgets[0].events[0].code).toEqual('args.owner.newRow();');
		expect(mergedContainer.widgets[0].fields.length).toEqual(2);
		expect(mergedContainer.widgets[0].fields[0].label).toEqual('Preference label');
		expect(mergedContainer.widgets[0].fields[1].name).toEqual('NEW_FIELD');
		expect(mergedContainer.widgets[0].fields[1].label).toEqual('New field');
		expect(mergedContainer.widgets[0].styles[0].cssClass).toEqual('zh-inactive');
		expect(mergedContainer.widgets[0].fieldGroups[0].name).toEqual('ORG changed');
		expect(mergedContainer.widgets[0].fieldGroups[0].opened).toEqual(false);
		expect(mergedContainer.widgets[0].widgets.length).toEqual(2);
		expect(mergedContainer.widgets[0].widgets[0].id).toEqual(100);
		expect(mergedContainer.widgets[0].widgets[0].name).toEqual('secondWidget');
		expect(mergedContainer.widgets[0].widgets[0].label).toEqual('2');
		expect(mergedContainer.widgets[0].widgets[0].fields.length).toEqual(1);
		expect(mergedContainer.widgets[0].widgets[0].events.length).toEqual(1);
		expect(mergedContainer.widgets[0].widgets[0].events[0].id).toEqual(102);
		expect(mergedContainer.widgets[0].widgets[0].events[0].code).toEqual('args.owner.newRow();');
		expect(mergedContainer.widgets[0].widgets[1].id).toEqual(1000);
		expect(mergedContainer.widgets[0].widgets[1].name).toEqual('thirdWidget');
		expect(mergedContainer.widgets[0].widgets[1].label).toEqual('3');
	});

	it("The method merge should merge two containers keeping the origin property like a backup", function() {
		var backupProperty = "bkp";
		var mergedContainer = containerMerger.merge(Util.clone(defaultContainer), Util.clone(defaultContainerPreference), backupProperty);
		expect(mergedContainer.showFooter_bkp).toEqual(true);
		expect(mergedContainer.showFooter).toEqual(false);
		expect(mergedContainer.showMenu).toEqual(true);
		expect(mergedContainer.widgets[0].template_bkp).toEqual('widget/list.html');
		expect(mergedContainer.widgets[0].template).toEqual('widget/form.html');
		expect(mergedContainer.widgets[0].actions[0].showAsAction_bkp).toEqual('always');
		expect(mergedContainer.widgets[0].actions[0].showAsAction).toEqual('never');
		expect(mergedContainer.widgets[0].actions[0].events[0].code_bkp).toEqual(';');
		expect(mergedContainer.widgets[0].actions[0].events[0].code).toEqual('args.owner.widget.newRow();');
		expect(mergedContainer.widgets[0].events[0].code_bkp).toEqual(';');
		expect(mergedContainer.widgets[0].events[0].code).toEqual('args.owner.newRow();');
		expect(mergedContainer.widgets[0].fields.length).toEqual(2);
		expect(mergedContainer.widgets[0].fields[0].label_bkp).toEqual('Default label');
		expect(mergedContainer.widgets[0].fields[0].label).toEqual('Preference label');
		expect(mergedContainer.widgets[0].fields[1].name).toEqual('NEW_FIELD');
		expect(mergedContainer.widgets[0].fields[1].label).toEqual('New field');
		expect(mergedContainer.widgets[0].styles[0].cssClass_bkp).toEqual('zh-active');
		expect(mergedContainer.widgets[0].styles[0].cssClass).toEqual('zh-inactive');
		expect(mergedContainer.widgets[0].fieldGroups[0].name_bkp).toEqual('ORG');
		expect(mergedContainer.widgets[0].fieldGroups[0].name).toEqual('ORG changed');
		expect(mergedContainer.widgets[0].fieldGroups[0].opened_bkp).toEqual(true);
		expect(mergedContainer.widgets[0].fieldGroups[0].opened).toEqual(false);
		expect(mergedContainer.widgets[0].widgets.length).toEqual(2);
		expect(mergedContainer.widgets[0].widgets[0].id).toEqual(100);
		expect(mergedContainer.widgets[0].widgets[0].name).toEqual('secondWidget');
		expect(mergedContainer.widgets[0].widgets[0].label_bkp).toEqual('Second widget');
		expect(mergedContainer.widgets[0].widgets[0].label).toEqual('2');
		expect(mergedContainer.widgets[0].widgets[0].fields.length).toEqual(1);
		expect(mergedContainer.widgets[0].widgets[0].events.length).toEqual(1);
		expect(mergedContainer.widgets[0].widgets[0].events[0].id).toEqual(102);
		expect(mergedContainer.widgets[0].widgets[0].events[0].code_bkp).toEqual(';');
		expect(mergedContainer.widgets[0].widgets[0].events[0].code).toEqual('args.owner.newRow();');
		expect(mergedContainer.widgets[0].widgets[1].id).toEqual(1000);
		expect(mergedContainer.widgets[0].widgets[1].name).toEqual('thirdWidget');
		expect(mergedContainer.widgets[0].widgets[1].label).toEqual('3');
	});

	it("The restore method should do a rollback in the properties of the merged container", function() {
		var backupProperty = "bkp";
		var mergedContainer = containerMerger.merge(Util.clone(defaultContainer), Util.clone(defaultContainerPreference), backupProperty);
		containerMerger.restore(mergedContainer, backupProperty);
		expect(mergedContainer.showFooter).toEqual(true);
		expect(mergedContainer.widgets[0].template).toEqual('widget/list.html');
		expect(mergedContainer.widgets[0].actions[0].showAsAction).toEqual('always');
		expect(mergedContainer.widgets[0].actions[0].events[0].code).toEqual(';');
		expect(mergedContainer.widgets[0].events[0].code).toEqual(';');
		expect(mergedContainer.widgets[0].fields[0].label).toEqual('Default label');
		expect(mergedContainer.widgets[0].styles[0].cssClass).toEqual('zh-active');
		expect(mergedContainer.widgets[0].fieldGroups[0].name).toEqual('ORG');
		expect(mergedContainer.widgets[0].fieldGroups[0].opened).toEqual(true);
		expect(mergedContainer.widgets[0].widgets[0].label).toEqual('Second widget');
		expect(mergedContainer.widgets[0].widgets[0].events[0].code).toEqual(';');
	});
});