describe('RepositoryFactory', function() {

	var buildMock = function() {
		var mock = {
			whenSuccess: function() {},
			whenError: function() {},
			eventError: function() {},
			eventSuccess: function() {},
			eventRetry: function() {},
			eventStart: function() {},
			eventEnd: function() {},
		};
		var event = {
			isPropagationStoped: function() {
				return false;
			}
		};
		spyOn(mock, 'whenSuccess').and.callThrough();
		spyOn(mock, 'whenError').and.callThrough();
		spyOn(mock, 'eventError').and.callThrough();
		spyOn(mock, 'eventSuccess').and.callThrough();
		spyOn(mock, 'eventRetry').and.callThrough();
		spyOn(mock, 'eventStart').and.callThrough();
		spyOn(mock, 'eventEnd').and.callThrough();
		EventAggregator.onRequestSuccess(mock.eventSuccess, event);
		EventAggregator.onRequestStart(mock.eventStart, event);
		EventAggregator.onRequestEnd(mock.eventEnd, event);
		EventAggregator.onRequestError(mock.eventError, event);
		EventAggregator.onRequestRetry(mock.eventRetry, event);
		return mock;
	};
	var controller;
	var repository;
	var repositoryRetry;
	var $rootScope;
	var $httpBackend;
	var EventAggregator;
	Configuration(function(RepositoryFactory, ContextRegister, ZhLogger, eventAggregator, templateManager) {
		repository = RepositoryFactory.factory("/repository", "LOCAL");
		EventAggregator = eventAggregator;
		ZhLogger.error = function() {}; //ShutUp idiot erros
		templateManager.container = {
			name: 'container',
			getActiveWidget: function() {
				return {
					name: 'widget'
				};
			},
			updateCheckedRows: function() {

			}
		};
	});
	Configuration(function(RepositoryFactory, ContextRegister) {
		repositoryRetry = RepositoryFactory.factory("/repositoryRetry", "LOCAL", 3, 100);
	});
	beforeEach(function() {
		window.templateUrl = "../../dyman/templates/dyman/";
		window.metadataUrl = "/json_test/test";
		window.metadataUrls = [];
		window.serviceUrl = "../backend/service/index.php";
		window.serviceUrls = [];
		module('ZeedhiApp');
	});
	describe(" Repository download", function() {
		beforeEach(inject(function(_$controller_, _$rootScope_, _$httpBackend_) {
			$rootScope = _$rootScope_;
			$httpBackend = _$httpBackend_;
			// The injector unwraps the underscores (_) from around the parameter names when matching
			$controller = _$controller_;
			var $scope = $rootScope;
			controller = $controller('ZeedhiMain', {
				$scope: $scope
			});
			$httpBackend.when('GET', '/json_test/test/json/acl.json').respond({}, {});
			$httpBackend.when('GET', '/json_test/test/json/words/pt_br.json').respond({}, {});
			$httpBackend.when('GET', '/json_test/test/json/data.json').respond({}, {});
			$httpBackend.when('GET', "/json_test/test/json/datasources/repository.json").respond({
				repository: {
					localKey: ['id']
				}
			}, {});
			$httpBackend.when('GET', "/json_test/test/json/containers.json").respond(ZhHelper.projectTest(), {});
			$httpBackend.when('GET', "/json_test/test/json/environment.json").respond(ZhHelper.emptyEnvironment, {});
			$httpBackend.when('GET', "/json_test/test/json/containers/login.json").respond(
				ZhHelper.emptyContainer, {});
			$httpBackend.when('GET', "/json_test/test/json/datasources/repositoryRetry.json").respond({
				repositoryRetry: {
					localKey: ['id']
				}
			}, {});
			$httpBackend.when('POST', "../backend/service/index.php/repository").respond(ZhHelper.repositoryRequestMock(
				"repository"), {});
			$httpBackend.when('GET', "/json_test/test/json/datasources/zhPreferences.json").respond(200, {});

			/* bower.json request (because of templateManager.updateVersionData()) */
			$httpBackend.when('GET', './bower.json').respond({}, {});

			//inject();
			$rootScope.$apply();
		}));
		it('Repository do not be undefined', function() {
			expect(repository).not.toEqual(undefined);
		});
		it('Repository download success need will be caled in a simple request', function(done) {
			$httpBackend.expectPOST("../backend/service/index.php/repository", undefined).respond(200, ZhHelper.repositoryRequestMock('repository'));
			var mock = buildMock();
			ZhHelper.doAsyncTest(function(_done) {
				var promise = repository.download();

				promise.then(function(){
					mock.whenSuccess();
				}, function(){
					mock.whenError();
				});
				Util.doAsync(function(){
					ZhHelper.securityApply($rootScope);
				}, 100);
				ZhHelper.securityApply($rootScope);
				ZhHelper.securityApply($rootScope);
				ZhHelper.securityApply($rootScope);
				ZhHelper.securityApply($rootScope);
				ZhHelper.securityApply($rootScope);
				ZhHelper.securityApply($rootScope);
			}, function() {
				ZhHelper.securityApply($rootScope);
				ZhHelper.securityApply($rootScope);
				ZhHelper.securityApply($rootScope);
				Util.doAsync(function() {
					ZhHelper.securityApply($rootScope);
					Util.doAsync(function() {
						ZhHelper.securityApply($rootScope);
						ZhHelper.securityApply($rootScope);
						expect(mock.whenSuccess).toHaveBeenCalled();
						expect(mock.whenError).not.toHaveBeenCalled();
						done();
					});
				});
			}, 1000, $rootScope, $httpBackend);
		});

		it('Repository download erro should be called when server repond with error', function(done) {
			$httpBackend.expectPOST("../backend/service/index.php/repository", undefined).respond(500, '');
			var mock = buildMock();
			ZhHelper.doAsyncTest(function(_done) {
				var promise = repository.download();
				promise.then(mock.whenSuccess).then(null, mock.whenError);
			}, function() {
				expect(mock.whenSuccess).not.toHaveBeenCalled();
				expect(mock.whenError).toHaveBeenCalled();
				expect(mock.eventSuccess).not.toHaveBeenCalled();
				expect(mock.eventError).toHaveBeenCalled();
				expect(mock.eventStart).toHaveBeenCalled();
				expect(mock.eventEnd).toHaveBeenCalled();
				done();
			}, 1000, $rootScope, $httpBackend);
		});
		it(
			'Repository retry download erro should be called when server repond with error after 3 times',
			function(done) {
				var count = 0;
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry").respond(500, '');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry").respond(500, '');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry").respond(500, '');
				var mock = buildMock();

				ZhHelper.doAsyncTest(function(_done) {
					var promise = repositoryRetry.download();
					repositoryRetry.onRetry(function() {
						ZhHelper.securityApply($rootScope);
						count++;
					});
					promise.then(mock.whenSuccess).then(null, mock.whenError);
				}, function() {
					expect(mock.whenSuccess).not.toHaveBeenCalled();
					expect(mock.whenError).toHaveBeenCalled();
					expect(mock.eventSuccess).not.toHaveBeenCalled();
					expect(mock.eventError).toHaveBeenCalled();
					expect(mock.eventRetry.calls.count()).toBe(2);

					expect(mock.eventStart).toHaveBeenCalled();
					expect(mock.eventEnd).toHaveBeenCalled();
					expect(mock.eventStart.calls.count()).toBe(3);
					expect(mock.eventEnd.calls.count()).toBe(3);

					expect(count).toBe(2);
					done();
				}, 1000, $rootScope, $httpBackend);
			});
		it(
			'Repository retry download erro should be called when server repond with error after 5 times',
			function(done) {
				repositoryRetry.retryCount = 5;
				var count = 0;
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry").respond(500, '');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry").respond(500, '');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry").respond(500, '');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry").respond(500, '');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry").respond(500, '');
				var mock = buildMock();
				ZhHelper.doAsyncTest(function(_done) {
					var promise = repositoryRetry.download();
					repositoryRetry.onRetry(function() {
						ZhHelper.securityApply($rootScope);
						count++;
					});
					promise.then(mock.whenSuccess).then(null, mock.whenError);
				}, function() {
					expect(mock.whenSuccess).not.toHaveBeenCalled();
					expect(mock.whenError).toHaveBeenCalled();
					expect(mock.eventSuccess).not.toHaveBeenCalled();
					expect(mock.eventError).toHaveBeenCalled();

					expect(mock.eventStart).toHaveBeenCalled();
					expect(mock.eventEnd).toHaveBeenCalled();
					expect(mock.eventStart.calls.count()).toBe(5);
					expect(mock.eventEnd.calls.count()).toBe(5);
					expect(mock.eventRetry.calls.count()).toBe(4);
					expect(count).toBe(4);
					done();
				}, 1000, $rootScope, $httpBackend);
			});
		it(
			'Repository retry saveInServer erro should be called when server repond with error after 5 times',
			function(done) {
				repositoryRetry.retryCount = 5;
				var count = 0;
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				var mockInner = buildMock();
				ZhHelper.doAsyncTest(function(_done) {
					var promise = repositoryRetry.saveInServer([{
						changed: true,
						name: "test"
					}]);
					repositoryRetry.onRetry(function() {
						ZhHelper.securityApply($rootScope);
						count++;
					});
					$rootScope.$apply();
					promise.then(mockInner.whenSuccess).then(null, mockInner.whenError);
				}, function() {
					$httpBackend.flush();
					expect(mockInner.whenSuccess).not.toHaveBeenCalled();
					expect(mockInner.whenError).toHaveBeenCalled();
					expect(mockInner.eventSuccess).not.toHaveBeenCalled();
					expect(mockInner.eventError).toHaveBeenCalled();

					expect(mockInner.eventStart).toHaveBeenCalled();
					expect(mockInner.eventEnd).toHaveBeenCalled();
					expect(mockInner.eventStart.calls.count()).toBe(5);
					expect(mockInner.eventEnd.calls.count()).toBe(5);

					expect(mockInner.eventRetry.calls.count()).toBe(4);
					expect(count).toBe(4);
					done();
				}, 3000, $rootScope, $httpBackend, false);
				$rootScope.$apply();
			});
		it('Repository retry saveInServer need call sucess when 4 time request respond sucess',
			function(done) {
				repositoryRetry.retryCount = 5;
				var count = 0;
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(200,
					'');
				var mockInner = buildMock();
				ZhHelper.doAsyncTest(function(_done) {
					var promise = repositoryRetry.saveInServer([{
						changed: true,
						name: "test"
					}]);
					repositoryRetry.onRetry(function() {
						ZhHelper.securityApply($rootScope);
						count++;
					});
					$rootScope.$apply();
					promise.then(mockInner.whenSuccess).then(null, mockInner.whenError);
				}, function() {
					$httpBackend.flush();
					expect(mockInner.whenSuccess).toHaveBeenCalled();
					expect(mockInner.whenError).not.toHaveBeenCalled();
					expect(mockInner.eventSuccess).toHaveBeenCalled();
					expect(mockInner.eventError).not.toHaveBeenCalled();
					expect(mockInner.eventRetry.calls.count()).toBe(4);

					expect(mockInner.eventStart).toHaveBeenCalled();
					expect(mockInner.eventEnd).toHaveBeenCalled();
					expect(mockInner.eventStart.calls.count()).toBe(5);
					expect(mockInner.eventEnd.calls.count()).toBe(5);

					expect(count).toBe(4);
					done();
				}, 3000, $rootScope, $httpBackend, false);
				$rootScope.$apply();
			});
		it('Repository retry saveInServer error need call sucess where the 4 time need sucess',
			function(done) {
				repositoryRetry.retryCount = 5;
				var count = 0;
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(500,
					'');
				$httpBackend.expectPOST("../backend/service/index.php/repositoryRetry/save").respond(200,
					'', {});
				var mockInner = buildMock();
				ZhHelper.doAsyncTest(function(_done) {
					var promise = repositoryRetry.saveInServer([{
						changed: true,
						name: "test"
					}]);
					repositoryRetry.onRetry(function() {
						ZhHelper.securityApply($rootScope);
						count++;
					});
					$rootScope.$apply();
					promise.then(mockInner.whenSuccess).then(null, mockInner.whenError);
				}, function() {
					$httpBackend.flush();
					expect(mockInner.whenSuccess).toHaveBeenCalled();
					expect(mockInner.whenError).not.toHaveBeenCalled();
					expect(mockInner.eventSuccess).toHaveBeenCalled();
					expect(mockInner.eventError).not.toHaveBeenCalled();
					expect(mockInner.eventRetry.calls.count()).toBe(4);
					expect(mockInner.eventStart).toHaveBeenCalled();
					expect(mockInner.eventEnd).toHaveBeenCalled();
					expect(mockInner.eventStart.calls.count()).toBe(5);
					expect(mockInner.eventEnd.calls.count()).toBe(5);
					expect(count).toBe(4);
					done();
				}, 3000, $rootScope, $httpBackend, false);
				$rootScope.$apply();
			});
	});
});