module.exports = function(grunt) {
	var testLibs = ["bower_components/angular-mocks/angular-mocks.js", "bower_components/q/q.js"];
	var srcPath = [
		'src/CarouselWidgetCreator.js',
		'src/LayoutPreferences.js'
	];
	var dependenciesPath = ['bower_components/zeedhi-frontend/assets/dist/main.js'];
	var specsPath = 'src/specs/*spec*.js';

	grunt.initConfig({
		concat: {
			zh: {
				options: {
					process: function(src, filepath) {
						return '\n' + '// FILE: ' + filepath + '\n' + src;
					}
				},
				src: [srcPath],
				dest: 'dist/zhLayoutPreferences.js'
			}
		},
		jshint: {
			all: ['Gruntfile.js', specsPath, srcPath]
		},
		jasmine: {
			pivotal: {
				// Your project's source files
				src: [srcPath],
				// Your Jasmine spec files
				options: {
					vendor: [dependenciesPath, testLibs],
					specs: specsPath,
					// Your spec helper files
					helpers: [],
					template: require('grunt-template-jasmine-istanbul'),
					templateOptions: {
						coverage: 'bin/coverage/coverage.json',
						report: 'bin/coverage',
						thresholds: {
							lines: 10,
							statements: 10,
							branches: 10,
							functions: 10
						}
					}
				}
			},
			toWatch: {
				// Your project's source files
				src: [srcPath],
				// Your Jasmine spec files
				options: {
					vendor: [dependenciesPath, testLibs],
					specs: specsPath,
					// Your spec helper files
					helpers: []
				}
			}
		},
		watch: {
			pivotal: {
				files: [specsPath, srcPath],
				tasks: ['jshint', 'concat']
			}
		},
		bump: {
			options: {
				files: ['bower.json', 'package.json'],
				pushTo: 'origin',
				commitFiles: ["-a"],
				push: true
			}
		},
		uglify: {
			options: {
				mangle: false,
				compress: false,
				report: 'min',
				// the banner is inserted at the top of the output
				banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
			},
			dist: {
				files: {
					'dist/zhLayoutPreferences.min.js': [srcPath]
				}
			}
		},
		buildEnvironment: {
			development: {
				configFile: 'example/config/development.json'
			},
			deploy: {
				configFile: 'example/config/deploy.json'
			},
			srcFolders: ['example/frontend', 'example/frontend/tests']
		},
		zhIdGenerator: {
			options: {
				jsonPath: ['./example/frontend/json'],
				replaceIds: false
			}
		}
	});

	grunt.loadNpmTasks('grunt-bump');
	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-jasmine');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-zh-environment-builder');
	grunt.loadNpmTasks('grunt-zh-id-generator');

	// Default task.
	grunt.registerTask('default', ['jshint', 'jasmine:pivotal', 'concat', 'uglify']);
	grunt.registerTask('merge', ['jshint', 'jasmine:toWatch', 'concat']);

};